
/*
 * bigint class definition
 */

#ifndef __BIGINT_H__
#define __BIGINT_H__

/*
 * fixed length of bigints. number of possible digits is BIGLENGTH * 4,
 * as each element holds four digits. memory usage is BIGLENGTH * 2.
 * BIGLENGTH should be defined as static const member value
 */

#define BIGLENGTH	10000

class bigint {
 public:
  /*
   * constructors and destructor
   */
  bigint ();
  bigint (const bigint &);
  bigint (const char * const);
  ~bigint();

  /*
   * input and output
   */

  friend ostream &operator<<	(ostream &, const bigint &);

  /*
   * operators
   */
  bigint &	operator=	(const bigint &);
  bigint &	operator=	(const long);
  bigint &	operator=	(const char * const);

  /*
   * native functions
   */

  bigint	operator*	(const bigint &) const;
  bigint	operator*	(const long)     const;
  bigint	operator/	(const bigint &) const;

  bigint &	operator*=	(const long);
  bigint &	operator+=	(const bigint &);
  bigint &	operator-=	(const bigint &);
  bigint &	operator+=	(const long);
  bigint &	operator-=	(const long);

  int		operator<	(const bigint &) const;
  int		operator>	(const bigint &) const;
  int		operator==	(const bigint &) const;
  int		operator<=	(const bigint &) const;
  int		operator>=	(const bigint &) const;

  int		operator==	(const long) const;

  bigint	operator%	(const bigint &) const;

  bigint &	operator++	(void);
  bigint &	operator--	(void);
  bigint &	operator-	(void);

  /*
   * helper functions
   */
  bigint	operator+	(const bigint &) const;
  bigint	operator-	(const bigint &) const;

  bigint &	operator*=	(const bigint &);
  bigint &	operator/=	(const bigint &);
  bigint &	operator%=	(const bigint &);

  int		operator!=	(const bigint &) const;
  int		operator!=	(const long) const;

  friend bigint operator*	(const long, const bigint &);
  friend bigint operator+	(const long, const bigint &);
  friend bigint operator-	(const long, const bigint &);

  friend int	operator==	(const long, const bigint &);
  friend int	operator!=	(const long, const bigint &);

 private:
  /*
   * private functions
   */
  bigint &	adjust		(void);
  bigint &	add_unsigned	(const bigint &);
  bigint &	sub_unsigned	(const bigint &);
  int		cmp_unsigned	(const bigint &) const;
  bigint &	addi_unsigned	(const long);
  bigint &	shiftleft	(const long);
  bigint &	mult_nnnn	(const long);

  /*
   * data
   */
  long length;
  long first;
  long sign;
  unsigned short digits[BIGLENGTH];
};

inline bigint bigint::operator+ (const bigint &other) const
{
  bigint result(*this);
  return (result += other);
}

inline bigint bigint::operator- (const bigint &other) const
{
  bigint result(*this);
  return (result -= other);
}

inline bigint operator* (const long zahl, const bigint &other)
{
  bigint result(other);
  return (result *= zahl);
}

inline bigint &bigint::operator*= (const bigint &other)
{
  bigint temp;
  temp  = *this * other;
  *this = temp;
  return *this;
}

inline bigint &bigint::operator/= (const bigint &other)
{
  bigint temp;
  temp  = *this / other;
  *this = temp;
  return *this;
}

inline bigint &bigint::operator%= (const bigint &other)
{
  bigint temp;
  temp  = *this % other;
  *this = temp;
  return *this;
}

inline int bigint::operator!= (const bigint &other) const
{
  return ! operator== (other);
}

inline int operator== (const long zahl, const bigint &other)
{
  return other.operator== (zahl);
}

inline int bigint::operator!= (const long zahl) const
{
  return ! operator== (zahl);
}

inline int operator!= (const long zahl, const bigint &other)
{
  return other.operator!= (zahl);
}

#endif /* __BIGINT_H__ */
